@@ -1,5 +1,5 @@
 # -*- perl -*-
-# $Id: RobotPUA.pm,v 1.1 1998/03/06 04:59:11 marc Exp $
+# $Id: RobotPUA.pm,v 1.3 2000/04/20 14:49:09 langhein Exp $
 
 package LWP::RobotPUA;
 use Exporter();
@@ -19,6 +19,13 @@ __END__
 
 LWP::RobotPUA - Parallel LWP::RobotUA
 
+=head1 SYNOPSIS
+
+  require LWP::RobotPUA;
+  $ua = new LWP::RobotPUA 'my-robot/0.1', 'me@foo.com';
+
+  (see description of LWP::Parallel::RobotUA)
+
 =head1 DESCRIPTION
 
 RobotPUA is a simple frontend to the LWP::Parallel::RobotUA
@@ -41,5 +48,12 @@ Marc Langheinrich, marclang@cs.washington.edu
 
 L<LWP::Parallel::RobotUA>, L<LWP::Parallel::UserAgent>, L<LWP::RobotUA>
 
+=head1 COPYRIGHT
+
+Copyright 1997-2000 Marc Langheinrich.
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
 =cut
 
